#include "testApp.h"
#include "particle.h"

#include "ofMain.h"

// comparison routine for sort...
bool comparisonFunction(  particle * a, particle * b ) { 
	return a->pos.x < b->pos.x; 
}              
      

//--------------------------------------------------------------
void testApp::setup(){
	ofBackground(255,255,255);
	stagebg.loadImage("bg.png");
	bg.loadImage("particle1.png");
    for (int i = 0; i < 500; i++){
        particle * p = new particle();
		p->bg = &bg;
		myParticles.push_back(p);
		
        myParticles[i]->pos.set(ofRandom(0,ofGetWidth()),ofRandom(0,ofGetHeight()),0);
        myParticles[i]->vel.set(0,0,0);
    }
	
	music.loadSound("1.mp3");
	music.play();
	music.setVolume(.3);
	
	fftSmoothed = new float[8192];
	for (int i = 0; i < 8192; i++)
	{
		fftSmoothed[i] = 0;
	}
	
	nBandsToGet = 256;
	avg			= 0;
	bRepel		= true;
	radius		= 40;
	strength	= 0.9f; 
	prevRad     = 0;

	
	panel.setup("control", 770, 0, 300, 400);
	panel.addPanel("flocking", 1, false);
	panel.setWhichPanel("flocking");
	panel.setWhichColumn(0);
	panel.addSlider("cohesion strength", "COHESION_STRENGTH", 0.01,0,1, false);
	panel.addSlider("cohesion radius", "COHESION_RADIUS", 50,0,200, false);
	panel.addSlider("alignment strength", "ALIGNMENT_STRENGTH", 0.01,0,1, false);
	panel.addSlider("alignment radius", "ALIGNMENT_RADIUS",  50,0,200, false);
	panel.addSlider("seperation strength", "SEPERATION_STRENGTH", 0.01,0, 1, false);
	panel.addSlider("seperation radius", "SEPERATION_RADIUS",  50,0,200, false);
	
	panel.addSlider("damping", "DAMPING", 0.01, 0.001, 0.5, false);
	
	panel.loadSettings("settings.xml");

	ofSetCircleResolution(30);
	ofSetRectMode(OF_RECTMODE_CENTER);
}

//--------------------------------------------------------------
void testApp::update(){
	

	panel.update();
	
	for (int i = 0; i <    myParticles.size(); i++){
		   myParticles[i]->cohesion.distance = panel.getValueF("COHESION_RADIUS");
		   myParticles[i]->alignment.distance = panel.getValueF("ALIGNMENT_RADIUS");
		   myParticles[i]->seperation.distance = panel.getValueF("SEPERATION_RADIUS");
		
		
		myParticles[i]->cohesion.strength = panel.getValueF("COHESION_STRENGTH");
	   myParticles[i]->alignment.strength = panel.getValueF("ALIGNMENT_STRENGTH");
		myParticles[i]->seperation.strength = panel.getValueF("SEPERATION_STRENGTH");
		   myParticles[i]->damping = panel.getValueF("DAMPING");
		
	}


	avg = 0;
	float * val = ofSoundGetSpectrum(nBandsToGet);		
	for (int i = 0;i < nBandsToGet; i++)
	{
		fftSmoothed[i] *= 0.9f;
		if (fftSmoothed[i] < val[i]) fftSmoothed[i] = val[i];
		avg += fftSmoothed[i];
	}
	
	avg = avg/(float)nBandsToGet;
	float move = ofMap(avg, 0.0,0.01,20,150,true);
	
	if (prevRad < radius)
	{
		bRepel = false;
	}
	else 
	{
		bRepel = true;
	}
	


	prevRad = radius;
	radius = move;
	float newStrength = ofMap(abs(radius-prevRad), 0,1.5, 0.2, 0.9,false);
	strength = newStrength;

	
	
	
	//ofBackground(0,0,0); //Grey background, NY style
	
    for (int i = 0; i < myParticles.size(); i++){
        myParticles[i]->resetForce();
    }
	
	for (int i = 0; i < myParticles.size(); i++){
		for (int j = 0; j < myParticles.size(); j++){
			if (i != j){
				myParticles[i]->addForFlocking(*myParticles[j]);	
			}
		}
	}
	
	float it = 0;
	for (int i = 0; i < myParticles.size(); i++){
		for (int j = i-1; j >= 0; j--){
			it = ofMap(i, 0, myParticles.size(), 0, nBandsToGet, true);
			myParticles[i]->addRepulsionForce( *myParticles[j],10,avg*50);
		}
    }
	
	 it = 0;
    for (int i = 0; i < myParticles.size(); i++){
		it = ofMap(i, 0, myParticles.size(), 0, nBandsToGet, true);
		float push = ofMap(fftSmoothed[(int)it],0, 0.03, 1,50);
		float mulitplier = ofMap(i, 0 , myParticles.size(),1,20);
		myParticles[i]->addFlockingForce();
		myParticles[i]->addAttractionForce( ofGetWidth()/2,ofGetHeight()/2,30,1500,.5);
		myParticles[i]->addRepulsionForce( ofGetWidth()/2,ofGetHeight()/2,30,150,.5);
		myParticles[i]->addRepulsionForce( ofGetWidth()/2,ofGetHeight()/2,0,150,fabs(fftSmoothed[(int)it])*40);
		myParticles[i]->addDampingForce();
        myParticles[i]->update();
	}
}

//--------------------------------------------------------------
void testApp::draw(){
	ofSetColor(255,255,255);
	stagebg.setAnchorPoint(-1024/2,-768/2);
	stagebg.draw(0,0);
	//	panel.draw();

	ofSetColor(255,0, 0);
	ofDrawBitmapString(ofToString(ofGetFrameRate()), 20, 20);
	// then draw:
	
	for (int i = 0; i < myParticles.size(); i++){
		 myParticles[i]->draw();
	 }
	 
	ofSetColor(0, 0, 0);
	float width = (float)(2*128) / nBandsToGet;
	for (int i = 0;i < nBandsToGet; i++){
		float mulitplier = ofMap(i, 0 , nBandsToGet,1,20);

		// (we use negative height here, because we want to flip them
		// because the top corner is 0,0)
		ofRect(100+i*width,ofGetHeight()-100,width,-(fftSmoothed[i] * 200 * mulitplier));
	}
	

}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){
   
}
//--------------------------------------------------------------
void testApp::keyReleased  (int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){

}

void testApp::mouseDragged(int x, int y, int button){
	
	panel.mouseDragged(x,y,button);
	
	
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	
	panel.mousePressed(x,y,button);
	
}

//--------------------------------------------------------------
void testApp::mouseReleased(){
	
	panel.mouseReleased();
}
